/*
 *  This file is part of M-Index library: https://bitbucket.org/disalab/m-index
 *
 *  M-Index library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  M-Index library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with M-Index library.  If not, see <http://www.gnu.org/licenses/>.
 */
package mindex;


import messif.data.DataObject;
import messif.indexing.PrecomputedDistancesFilter;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Set;

/**
 * This filter checks that each data object is processed only once (this makes
 * sense only if multi-overlay M-Index used). TODO: Implement this functionality in a different way
 * 
 * @author David Novak, Masaryk University, Brno, Czech Republic, novak.david@gmail.com
 */
public class DuplicateRefineFilter extends PrecomputedDistancesFilter {

    public static final String FILTER_STRING = "duplicate_refine_filter".intern();

    private static final long serialVersionUID = 1L;

    protected final Object checkedObject;

    //protected final TObjectIntMap<Object> processedObjects;
    protected final Set<Object> processedObjects;

//    public void setProcessedObjects(THashSet processedObjects) {
//        this.processedObjects = processedObjects;
//    }
//    private transient int noEntryValue;

    /**
     * This is a constructor that is used for the data objects.
     * @param cheObject this is typically the {@link DataObject#getID()} field.
     */
    public DuplicateRefineFilter(Object cheObject) {
        this(cheObject, null);
    }

    /**
     * This constructor is used for the query object.
     * @param processedObjects this is typically an empty set in the beginning
     */
    public DuplicateRefineFilter(Set<Object> processedObjects) {
        this(null, processedObjects);
    }

    protected DuplicateRefineFilter(Object cheObject, Set<Object> processedObjects) {
        super(null);
        this.checkedObject = cheObject;
        this.processedObjects = processedObjects;
//        if (processedObjects != null) {
//            noEntryValue = processedObjects.getNoEntryValue();
//        }
    }

    @Override
    public boolean excludeUsingPrecompDist(PrecomputedDistancesFilter targetFilter, float radius) {
//        return noEntryValue != processedObjects.putIfAbsent(((DuplicateRefineFilter)targetFilter).checkedObject, 1);
        //synchronized (processedObjects) {
//            return null != processedObjects.putIfAbsent(((DuplicateRefineFilter)targetFilter).checkedObject, ((DuplicateRefineFilter)targetFilter).checkedObject);
            return ! processedObjects.add(((DuplicateRefineFilter)targetFilter).checkedObject);
        //}
    }

    // **************      The rest is not supported       ********************************** //

    @Override
    public boolean includeUsingPrecompDist(PrecomputedDistancesFilter targetFilter, float radius) {
        return false;
    }

    @Override
    protected boolean addPrecomputedDistance(DataObject obj, float distance, float[] metaDistances) {
        return false;
    }

    @Override
    public float getPrecomputedDistance(DataObject obj, float[] metaDistances) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    protected void writeData(OutputStream stream) throws IOException {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    protected boolean isDataWritable() {
        return false;
    }

}
