/*
 *  This file is part of M-Index library: https://bitbucket.org/disalab/m-index
 *
 *  M-Index library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  M-Index library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with M-Index library.  If not, see <http://www.gnu.org/licenses/>.
 */
package mindex;

import messif.data.DataObject;
import messif.distance.DistanceFunc;
import messif.indexing.IndexObject;

/**
 * Static methods to work with M-Index keys: PPP (encoded either as an array of shorts or as a single long value), and
 *  pivot-object distance.
 * 
 * @author David Novak, Masaryk University, Brno, Czech Republic, novak.david@gmail.com
 */
public class MIndexKeys {

    public static final String PPP = "ppp_shorts".intern();

    public static final String OBJ_PIVOT_DISTANCE = "objpivot_distance".intern();

    public static final String OBJ_PIVOT_DISTANCES = "objpivot_distances".intern();

    /**
     * Given an {@link messif.indexing.IndexObject object}, this method simply returns its {@link #PPP}, if it exists,
     *  null is returned otherwise.
     * @param object data object to read pivot permutation from
     * @return object PPP if it exists, null is returned otherwise
     */
    public static short [] readMIndexPP(DataObject object) {
        if (! (object instanceof IndexObject)) {
            return null;
        }
        return ((IndexObject) object).getIndexField(MIndexKeys.PPP, short[].class);
    }

    public static short readPivotIndex(DataObject object, int level) {
        short [] ppp = readMIndexPP(object);
        if ((ppp == null) || (level < 1 || level > ppp.length)) {
            return -1;
        }
        return ppp[level-1];
    }

    public static float readObjPivotDist(DataObject object) {
        if (! (object instanceof IndexObject)) {
            return DistanceFunc.UNKNOWN_DISTANCE;
        }
        return ((IndexObject) object).getIndexField(MIndexKeys.OBJ_PIVOT_DISTANCE, Float.class);
    }

    public static float [] readObjPivotDists(IndexObject object) {
        return ((IndexObject) object).getIndexField(MIndexKeys.OBJ_PIVOT_DISTANCES, float[].class);
    }


    /**
     * This method returns the number of cluster corresponding to this PP an level;
     *  it also depends on the maxlevel (size of PP).
     * @param level is 1-based
     * @return M-Index number of corresponding cluster
     */
//    private long computeClusterNumber(short [] pivotPermutation, int level, int maxLevel) {
//        long retVal = 0L;
//        long constant = 1L;
//        for (int i = maxLevel - 1; i >= 0; i--) {
//            if (i < level && i < ppLength) {
//                retVal += pivotPermutation[i] * constant;
//            }
//            constant *= nPivots;
//        }
//        return retVal;
//    }
}