/*
 *  This file is part of M-Index library: https://bitbucket.org/disalab/m-index
 *
 *  M-Index library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  M-Index library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with M-Index library.  If not, see <http://www.gnu.org/licenses/>.
 */
package mindex.privates;

import messif.utility.ExtendedProperties;
import mindex.MIndex;
import mindex.VoronoiTreeIndex;

import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;


/**
 *
 * @author xnovak8
 */
public class PrintPivotDistancesMatrix {

    private static void printUsage() {
        System.err.println("Usage: PrintPivotDistancesMatrix <mindex>[.properties]");
        System.err.println("   Creates the M-index (either deserialization or from .properties file)");
        System.err.println("   and calculates distances between all pairs of pivots and print them");
    }
    // M-Index config
    private static VoronoiTreeIndex mIndex = null;

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {

            /************ Parsing arguments ****************/
            int paramInd = 0;

            if (args.length < 1) {
                printUsage();
                return;
            }

            // Read MChord config
            String mIndexFile = args[paramInd++];
            if (mIndexFile.endsWith(".properties")) {
                ExtendedProperties extendedProperties = new ExtendedProperties();
                extendedProperties.load(new FileInputStream(new File(mIndexFile)));
                mIndex = new MIndex(extendedProperties, "mindex.");
            } else {
                ObjectInputStream configInput = new ObjectInputStream(new FileInputStream(mIndexFile));
                mIndex = (VoronoiTreeIndex) configInput.readObject();
                configInput.close();
            }

            System.err.print("Configuration: ");
            System.err.println(mIndex);

            StringBuffer buf = new StringBuffer("\\begin{tabular}{|r|");
            for (int i = 0; i < mIndex.getNumberOfPivots();  i++ ) {
                buf.append("c|");
            }
            buf.append("}\n\\hline\n & {\\bfseries 0}");
            for (int i = 1; i < mIndex.getNumberOfPivots(); i++) {
                buf.append(" & {\\bfseries ").append(i).append("}");
            }
            buf.append("\\\\ \\hline\\hline\n");
            for (int i = 0; i < mIndex.getNumberOfPivots(); i++) {
                buf.append("{\\bfseries ").append(i).append("}");
                for (int j = 0; j < mIndex.getNumberOfPivots(); j++) {
                    buf.append(" & ").append(String.format("%1.2f", mIndex.getPivotsDistance(i,j)));
                }
                buf.append("\\\\ \\hline\n");
            }
            buf.append("\\end{tabular}\n");

            System.out.println(buf.toString());

        } catch (Throwable e) {
            e.printStackTrace();
            printUsage();
        }
    }
}
