/*
 *  This file is part of M-Index library: https://bitbucket.org/disalab/m-index
 *
 *  M-Index library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  M-Index library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with M-Index library.  If not, see <http://www.gnu.org/licenses/>.
 */
package mindex.processor;


import messif.algorithm.AlgorithmMethodException;
import messif.operation.params.ApproximateSearch;
import messif.operation.search.KNNOperation;
import mindex.MIndex;

/**
 * Implementation of {@link messif.operation.processing.OperationProcessor} that processes the {@link KNNOperation}.
 * 
 * 
 * @author David Novak, Masaryk University, Brno, Czech Republic, novak.david@gmail.com
 */
public class KNNOperationProgressiveProcessor extends ApproxQueryProcessor {

    /**
     * Creates new approximate navigation processor for given operation and M-Index. The priority queue is initialized.
     * @param operation approximate kNN operation (should be the same for approximate range)
     * @param mIndex M-Index object with config and dynamic cell tree
     * @throws AlgorithmMethodException if the M-Index cannot determine the PP for query object
     */
    public KNNOperationProgressiveProcessor(KNNOperation operation, MIndex mIndex) {
        super(operation, new PreciseApproxSettings(), mIndex);
    }

    /** Class encapsulating the approximation settings */
    private static class PreciseApproxSettings implements ApproximateSearch {

        @Override
        public ApproximateSearch.Type getApproxType() {
            return Type.PERCENTAGE;
        }

        @Override
        public int getApproxParam() {
            return 100;
        }

        @Override
        public float getRadiusGuaranteed(float defaultRadius) {
            return defaultRadius;
        }


    }
}
